@@ -28,15 +28,15 @@ import (
 	"k8s.io/cli-runtime/pkg/resource"
 	corev1client "k8s.io/client-go/kubernetes/typed/core/v1"
 	watchtools "k8s.io/client-go/tools/watch"
-	"k8s.io/kubernetes/pkg/kubectl"
-	"k8s.io/kubernetes/pkg/kubectl/cmd/attach"
-	"k8s.io/kubernetes/pkg/kubectl/cmd/logs"
-	kcmdutil "k8s.io/kubernetes/pkg/kubectl/cmd/util"
-	"k8s.io/kubernetes/pkg/kubectl/polymorphichelpers"
-	"k8s.io/kubernetes/pkg/kubectl/scheme"
-	"k8s.io/kubernetes/pkg/kubectl/util/templates"
-	"k8s.io/kubernetes/pkg/kubectl/util/term"
-	"k8s.io/kubernetes/pkg/util/interrupt"
+	"k8s.io/kubectl/pkg/cmd/attach"
+	"k8s.io/kubectl/pkg/cmd/logs"
+	krun "k8s.io/kubectl/pkg/cmd/run"
+	kcmdutil "k8s.io/kubectl/pkg/cmd/util"
+	"k8s.io/kubectl/pkg/polymorphichelpers"
+	"k8s.io/kubectl/pkg/scheme"
+	"k8s.io/kubectl/pkg/util/interrupt"
+	"k8s.io/kubectl/pkg/util/templates"
+	"k8s.io/kubectl/pkg/util/term"
 
 	appsv1 "github.com/openshift/api/apps/v1"
 	dockerv10 "github.com/openshift/api/image/docker10"
@@ -79,7 +79,7 @@ var (
 		a pod (like a deployment, build, or job), objects that can host pods (like nodes), or
 		resources that can be used to create pods (such as image stream tags).
 
-		The debug pod is deleted when the the remote command completes or the user interrupts
+		The debug pod is deleted when the remote command completes or the user interrupts
 		the shell.`)
 
 	debugExample = templates.Examples(`
@@ -141,6 +141,7 @@ type DebugOptions struct {
 	DryRun             bool
 	FullCmdName        string
 	Image              string
+	ToNamespace        string
 
 	// IsNode is set after we see the object we're debugging.  We use it to be able to print pertinent advice.
 	IsNode bool
@@ -204,6 +205,7 @@ func NewCmdDebug(fullName string, f kcmdutil.Factory, streams genericclioptions.
 	cmd.Flags().BoolVar(&o.AsRoot, "as-root", o.AsRoot, "If true, try to run the container as the root user")
 	cmd.Flags().Int64Var(&o.AsUser, "as-user", o.AsUser, "Try to run the container as a specific user UID (note: admins may limit your ability to use this flag)")
 	cmd.Flags().StringVar(&o.Image, "image", o.Image, "Override the image used by the targeted container.")
+	cmd.Flags().StringVar(&o.ToNamespace, "to-namespace", o.ToNamespace, "Override the namespace to create the pod into (instead of using --namespace).")
 
 	o.PrintFlags.AddFlags(cmd)
 	kcmdutil.AddDryRunFlag(cmd)
@@ -343,17 +345,13 @@ func (o *DebugOptions) RunDebug() error {
 		return fmt.Errorf("you must identify a resource with a pod template to debug")
 	}
 
-	templateV1, err := o.approximatePodTemplateForObject(infos[0].Object)
-	if err != nil && templateV1 == nil {
+	template, err := o.approximatePodTemplateForObject(infos[0].Object)
+	if err != nil && template == nil {
 		return fmt.Errorf("cannot debug %s: %v", infos[0].Name, err)
 	}
 	if err != nil {
 		klog.V(4).Infof("Unable to get exact template, but continuing with fallback: %v", err)
 	}
-	template := &corev1.PodTemplateSpec{}
-	if err := scheme.Scheme.Convert(templateV1, template, nil); err != nil {
-		return err
-	}
 	pod := &corev1.Pod{
 		ObjectMeta: template.ObjectMeta,
 		Spec:       template.Spec,
@@ -362,6 +360,9 @@ func (o *DebugOptions) RunDebug() error {
 	if len(ns) == 0 {
 		ns = o.Namespace
 	}
+	if len(o.ToNamespace) > 0 {
+		ns = o.ToNamespace
+	}
 	pod.Name, pod.Namespace = fmt.Sprintf("%s-debug", generateapp.MakeSimpleName(infos[0].Name)), ns
 	o.Attach.Pod = pod
 
@@ -443,7 +444,7 @@ func (o *DebugOptions) RunDebug() error {
 
 		ctx, cancel := context.WithTimeout(context.Background(), o.Timeout)
 		defer cancel()
-		switch containerRunningEvent, err := watchtools.UntilWithoutRetry(ctx, w, conditions.PodContainerRunning(o.Attach.ContainerName)); {
+		switch containerRunningEvent, err := watchtools.UntilWithoutRetry(ctx, w, conditions.PodContainerRunning(o.Attach.ContainerName, o.CoreClient)); {
 		// api didn't error right away but the pod wasn't even created
 		case kapierrors.IsNotFound(err):
 			msg := fmt.Sprintf("unable to create the debug pod %q", pod.Name)
@@ -452,20 +453,17 @@ func (o *DebugOptions) RunDebug() error {
 			}
 			return fmt.Errorf(msg)
 			// switch to logging output
-		case err == kubectl.ErrPodCompleted, err == conditions.ErrContainerTerminated, !o.Attach.Stdin:
-			return logs.LogsOptions{
-				Object: pod,
-				Options: &corev1.PodLogOptions{
-					Container: o.Attach.ContainerName,
-					Follow:    true,
-				},
-				RESTClientGetter: o.RESTClientGetter,
-				ConsumeRequestFn: logs.DefaultConsumeRequest,
-				IOStreams:        o.IOStreams,
-				LogsForObject:    o.LogsForObject,
-			}.RunLogs()
+		case err == krun.ErrPodCompleted, err == conditions.ErrContainerTerminated:
+			return o.getLogs(pod)
+		case err == conditions.ErrNonZeroExitCode:
+			if err = o.getLogs(pod); err != nil {
+				return err
+			}
+			return conditions.ErrNonZeroExitCode
 		case err != nil:
 			return err
+		case !o.Attach.Stdin:
+			return o.getLogs(pod)
 		default:
 			// TODO this doesn't do us much good for remote debugging sessions, but until we get a local port
 			// set up to proxy, this is what we've got.
@@ -596,10 +594,6 @@ func (o *DebugOptions) getContainerImageCommand(pod *corev1.Pod, container *core
 func (o *DebugOptions) transformPodForDebug(annotations map[string]string) (*corev1.Pod, []string) {
 	pod := o.Attach.Pod
 
-	if !o.KeepInitContainers {
-		pod.Spec.InitContainers = nil
-	}
-
 	// reset the container
 	container := containerForName(pod, o.Attach.ContainerName)
 
@@ -664,10 +658,22 @@ func (o *DebugOptions) transformPodForDebug(annotations map[string]string) (*cor
 		container.SecurityContext.RunAsNonRoot = nil
 	}
 
-	if o.OneContainer {
+	switch {
+	case o.OneContainer:
+		pod.Spec.InitContainers = nil
 		pod.Spec.Containers = []corev1.Container{*container}
+	case o.KeepInitContainers:
+		// there is nothing we need to do
+	case isInitContainer(pod, o.Attach.ContainerName):
+		// keep only the init container we are debugging
+		pod.Spec.InitContainers = []corev1.Container{*container}
+	default:
+		// clear all init containers
+		pod.Spec.InitContainers = nil
 	}
 
+	clearHostPorts(pod)
+
 	// reset the pod
 	if pod.Annotations == nil || !o.KeepAnnotations {
 		pod.Annotations = make(map[string]string)
@@ -725,27 +731,64 @@ func (o *DebugOptions) createPod(pod *corev1.Pod) (*corev1.Pod, error) {
 }
 
 func containerForName(pod *corev1.Pod, name string) *corev1.Container {
-	for i, c := range pod.Spec.Containers {
-		if c.Name == name {
-			return &pod.Spec.Containers[i]
-		}
-	}
 	for i, c := range pod.Spec.InitContainers {
 		if c.Name == name {
 			return &pod.Spec.InitContainers[i]
 		}
 	}
+	for i, c := range pod.Spec.Containers {
+		if c.Name == name {
+			return &pod.Spec.Containers[i]
+		}
+	}
 	return nil
 }
 
+func isInitContainer(pod *corev1.Pod, name string) bool {
+	for _, c := range pod.Spec.InitContainers {
+		if c.Name == name {
+			return true
+		}
+	}
+	return false
+}
+
 func containerNames(pod *corev1.Pod) []string {
 	var names []string
+	for _, c := range pod.Spec.InitContainers {
+		names = append(names, c.Name)
+	}
 	for _, c := range pod.Spec.Containers {
 		names = append(names, c.Name)
 	}
 	return names
 }
 
+func clearHostPorts(pod *corev1.Pod) {
+	for i := range pod.Spec.InitContainers {
+		if pod.Spec.HostNetwork {
+			pod.Spec.InitContainers[i].Ports = nil
+			continue
+		}
+		for j := range pod.Spec.InitContainers[i].Ports {
+			if pod.Spec.InitContainers[i].Ports[j].HostPort > 0 {
+				pod.Spec.InitContainers[i].Ports[j].HostPort = 0
+			}
+		}
+	}
+	for i := range pod.Spec.Containers {
+		if pod.Spec.HostNetwork {
+			pod.Spec.Containers[i].Ports = nil
+			continue
+		}
+		for j := range pod.Spec.Containers[i].Ports {
+			if pod.Spec.Containers[i].Ports[j].HostPort > 0 {
+				pod.Spec.Containers[i].Ports[j].HostPort = 0
+			}
+		}
+	}
+}
+
 func (o *DebugOptions) approximatePodTemplateForObject(object runtime.Object) (*corev1.PodTemplateSpec, error) {
 	switch t := object.(type) {
 	case *corev1.Node:
@@ -916,7 +959,23 @@ func (o *DebugOptions) approximatePodTemplateForObject(object runtime.Object) (*
 	return nil, fmt.Errorf("unable to extract pod template from type %v", reflect.TypeOf(object))
 }
 
+func (o *DebugOptions) getLogs(pod *corev1.Pod) error {
+	return logs.LogsOptions{
+		Object: pod,
+		Options: &corev1.PodLogOptions{
+			Container: o.Attach.ContainerName,
+			Follow:    true,
+		},
+		RESTClientGetter: o.RESTClientGetter,
+		ConsumeRequestFn: logs.DefaultConsumeRequest,
+		IOStreams:        o.IOStreams,
+		LogsForObject:    o.LogsForObject,
+	}.RunLogs()
+}
+
 func setNodeName(template *corev1.PodTemplateSpec, nodeName string) *corev1.PodTemplateSpec {
-	template.Spec.NodeName = nodeName
+	if len(nodeName) > 0 {
+		template.Spec.NodeName = nodeName
+	}
 	return template
 }
